@@ -1,8 +1,24 @@
+;; This program is free software; you can redistribute it and/or    
+;; modify it under the terms of the GNU General Public License as   
+;; published by the Free Software Foundation; either version 2 of   
+;; the License, or (at your option) any later version.              
+;;                                                                  
+;; This program is distributed in the hope that it will be useful,  
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
+;; GNU General Public License for more details.                     
+;;                                                                  
+;; You should have received a copy of the GNU General Public License
+;; along with this program; if not, contact:
+;;
+;; Free Software Foundation           Voice:  +1-617-542-5942
+;; 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652
+;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 
 (define-module (gnucash app-utils))
-(use-modules (g-wrapped gw-app-utils))
+(use-modules (sw_app_utils))
 (use-modules (srfi srfi-1))
-(use-modules (gnucash main) (g-wrapped gw-gnc)) ;; FIXME: delete after we finish modularizing.
+(use-modules (gnucash main)) ;; FIXME: delete after we finish modularizing.
 (use-modules (gnucash gnc-module))
 
 (gnc:module-load "gnucash/engine" 0)
@@ -15,6 +31,7 @@
 (export-syntax N_)
 (export gnc:make-string-database)
 
+;; options.scm
 (export gnc:make-option)
 (export gnc:option-section)
 (export gnc:option-name)
@@ -52,6 +69,7 @@
 (export gnc:make-complex-boolean-option)
 (export gnc:make-pixmap-option)
 (export gnc:make-date-option)
+(export gnc:make-budget-option)
 (export gnc:get-rd-option-data-subtype)
 (export gnc:get-rd-option-data-show-time)
 (export gnc:get-rd-option-data-rd-list)
@@ -77,6 +95,8 @@
 (export gnc:make-internal-option)
 (export gnc:make-query-option)
 (export gnc:make-color-option)
+(export gnc:make-dateformat-option)
+(export gnc:dateformat-get-format)
 
 (export gnc:color->html)
 (export gnc:color-option->html)
@@ -116,16 +136,12 @@
 (export gnc:config-var-value-is-default?)
 
 ;; prefs.scm
-(export gnc:register-configuration-option)
-(export gnc:lookup-global-option)
-(export gnc:send-global-options)
-(export gnc:global-options-clear-changes)
-(export gnc:save-all-options)
 (export gnc:get-debit-string)
 (export gnc:get-credit-string)
-(export gnc:*options-entries*)
 (export gnc:config-file-format-version)
-(export gnc:*save-options-hook*)
+
+;; gw-engine-spec.scm
+(re-export HOOK-SAVE-OPTIONS)
 
 ;; date-utilities.scm
 
@@ -135,16 +151,24 @@
 (export gnc:timepair->date)
 (export gnc:date->timepair)
 (export gnc:date-get-year)
+(export gnc:date-get-quarter)
 (export gnc:date-get-month-day)
 (export gnc:date-get-month)
 (export gnc:date-get-week-day)
 (export gnc:date-get-year-day)
 (export gnc:timepair-get-year)
+(export gnc:timepair-get-quarter)
 (export gnc:timepair-get-month-day)
 (export gnc:timepair-get-month)
 (export gnc:timepair-get-week-day)
+(export gnc:timepair-get-week)
 (export gnc:timepair-get-year-day)
+(export gnc:date-get-year-string)
+(export gnc:date-get-quarter-string)
+(export gnc:date-get-quarter-year-string)
 (export gnc:date-get-month-string)
+(export gnc:date-get-month-year-string)
+(export gnc:date-get-week-year-string)
 (export gnc:leap-year?)
 (export gnc:days-in-year)
 (export gnc:days-in-month)
@@ -152,7 +176,9 @@
 (export gnc:date-year-delta)
 (export gnc:date-to-month-fraction)
 (export gnc:date-to-week-fraction)
+(export gnc:date-to-week)
 (export gnc:date-to-day-fraction)
+(export gnc:date-get-fraction-func)
 (export moddatek)
 (export decdate)
 (export incdate)
@@ -203,10 +229,6 @@
 (export gnc:get-end-cal-year)
 (export gnc:get-start-prev-year)
 (export gnc:get-end-prev-year)
-(export gnc:get-start-cur-fin-year)
-(export gnc:get-start-prev-fin-year)
-(export gnc:get-end-prev-fin-year)
-(export gnc:get-end-cur-fin-year)
 (export gnc:get-start-this-month)
 (export gnc:get-end-this-month)
 (export gnc:get-start-prev-month)
@@ -221,28 +243,21 @@
 (export gnc:get-six-months-ago)
 (export gnc:get-one-year-ago)
 (export gnc:reldate-initialize)
+(export gnc:get-end-next-month)
+(export gnc:get-end-next-quarter)
+(export gnc:get-end-next-year)
+(export gnc:get-one-month-ahead)
+(export gnc:get-one-year-ahead)
+(export gnc:get-six-months-ahead)
+(export gnc:get-start-next-month)
+(export gnc:get-start-next-quarter)
+(export gnc:get-start-next-year)
+(export gnc:get-three-months-ahead)
 
 ;; hooks 
-(export gnc:hook-define)
-(export gnc:hook-danglers-get)
-(export gnc:hook-danglers-set!)
-(export gnc:hook-danglers->list)
-(export gnc:hook-replace-danglers)
-(export gnc:hook-run-danglers)
-(export gnc:hook-lookup)
-(export gnc:hook-add-dangler)
-(export gnc:hook-remove-dangler)
-(export gnc:hook-description-get)
-(export gnc:hook-name-get)
-(export gnc:*startup-hook*)
-(export gnc:*shutdown-hook*)
-(export gnc:*ui-startup-hook*)
-(export gnc:*ui-post-startup-hook*)
-(export gnc:*ui-shutdown-hook*)
-(export gnc:*book-opened-hook*)
-(export gnc:*new-book-hook*)
-(export gnc:*book-closed-hook*)
-(export gnc:*report-hook*)
+(export gnc:hook-run-danglers)		;; from hooks.scm
+(re-export gnc-hook-add-scm-dangler)
+(re-export HOOK-REPORT)
 
 ;; simple-obj
 (export make-simple-class)
@@ -253,48 +268,14 @@
 (export simple-obj-from-list)
 (export make-simple-obj)
 
-;; kvp-option-registry
-(export gnc:register-kvp-option-generator)
-(export gnc:unregister-kvp-option-generator)
-(export gnc:make-kvp-options)
+(define gnc:*kvp-option-path* (list KVP-OPTION-PATH))
 (export gnc:*kvp-option-path*)
 
 (load-from-path "c-interface.scm")
 (load-from-path "config-var.scm")
 (load-from-path "options.scm")
-(load-from-path "kvp-option-registry.scm")
 (load-from-path "hooks.scm")
 (load-from-path "prefs.scm")
 (load-from-path "date-utilities.scm")
 (load-from-path "simple-obj.scm")
 
-
-
-(gnc:hook-add-dangler gnc:*startup-hook*
-                      (lambda ()
-                        (begin
-                          ;; Initialize the C side options code.
-                          ;; Must come after the scheme options are loaded.
-                          (gnc:c-options-init)
-
-                          ;; Initialize the expression parser.
-                          ;; Must come after the C side options initialization.
-                          (gnc:exp-parser-init))))
-
-;; add a hook to shut down the expression parser
-(gnc:hook-add-dangler gnc:*shutdown-hook*
-                      (lambda ()
-                        (begin
-                          ;; Shutdown the expression parser
-                          (gnc:exp-parser-shutdown)
-
-                          ;; This saves global options plus (for the
-                          ;; moment) saved report and account tree
-                          ;; window parameters. Reports and parameters
-                          ;; should probably be in a separate file,
-                          ;; with the main data file, or something
-                          ;; else.
-                          (gnc:save-all-options)
-
-                          ;; Shut down the C side options code
-                          (gnc:c-options-shutdown))))
