@@ -17,9 +17,27 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 \********************************************************************/
 
+#include <libguile/version.h> /* for SCM_MAJOR_VERSION etc */
+
+/* Give Guile 1.6 and 1.8 a 2.0-like interface */
+#if (SCM_MAJOR_VERSION == 1) && (SCM_MINOR_VERSION <= 6)
+# define scm_is_bool SCM_BOOLP
+# define scm_is_false SCM_FALSEP
+# define scm_is_null SCM_NULLP
+# define scm_is_number SCM_NUMBERP
+# define scm_is_pair SCM_CONSP
+# define scm_is_string SCM_STRINGP
+# define scm_is_symbol SCM_SYMBOLP
+# define scm_is_true SCM_NFALSEP
+# define scm_is_vector SCM_VECTORP
+# define scm_to_locale_string SCM_STRING_CHARS
+# define scm_c_string_length SCM_STRING_LENGTH
+#elif (SCM_MAJOR_VERSION == 1) && (SCM_MINOR_VERSION <= 8)
+# define scm_c_string_length scm_i_string_length
+#endif
 /* Convenience macros */
 
-#define SCM_EQUALP(obj1,obj2)	SCM_NFALSEP(scm_equal_p(obj1,obj2))
-#define SCM_EXACTP(obj)		SCM_NFALSEP(scm_exact_p(obj))
-#define SCM_LISTP(obj)		SCM_NFALSEP(scm_list_p(obj))
-#define SCM_PROCEDUREP(obj)	SCM_NFALSEP(scm_procedure_p(obj))
+#define scm_is_equal(obj1,obj2)	scm_is_true(scm_equal_p(obj1,obj2))
+#define scm_is_exact(obj)	scm_is_true(scm_exact_p(obj))
+#define scm_is_list(obj)	scm_is_true(scm_list_p(obj))
+#define scm_is_procedure(obj)	scm_is_true(scm_procedure_p(obj))
