@@ -38,6 +38,7 @@
 #include "dialog-ab-trans.h"
 #include "gnc-ab-kvp.h"
 #include "gnc-ab-utils.h"
+#include "gnc-gconf-utils.h"
 #include "gnc-glib-utils.h"
 #include "gnc-gwen-gui.h"
 #include "gnc-ui.h"
@@ -46,6 +47,10 @@
 #include "import-utilities.h"
 #include "qof.h"
 
+#ifdef AQBANKING_VERSION_5_PLUS
+# include <aqbanking/abgui.h>
+#endif /* AQBANKING_VERSION_5_PLUS */
+
 /* This static indicates the debugging module that this .o belongs to.  */
 static QofLogModule log_module = G_LOG_DOMAIN;
 
@@ -63,7 +68,8 @@ static AB_IMEXPORTER_ACCOUNTINFO *txn_accountinfo_cb(
 static AB_IMEXPORTER_ACCOUNTINFO *bal_accountinfo_cb(
     AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data);
 
-struct _GncABImExContextImport {
+struct _GncABImExContextImport
+{
     guint awaiting;
     gboolean txn_found;
     Account *gnc_acc;
@@ -73,6 +79,7 @@ struct _GncABImExContextImport {
     GtkWidget *parent;
     AB_JOB_LIST2 *job_list;
     GNCImportMainMatcher *generic_importer;
+    GData *tmp_job_list;
 };
 
 void
@@ -84,9 +91,18 @@ gnc_GWEN_Init(void)
     GWEN_Init();
 
     /* Initialize gwen logging */
-    GWEN_Logger_SetLevel(NULL, GWEN_LoggerLevel_Debug);
-    GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Debug);
-    GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Debug);
+    if (gnc_gconf_get_bool(GCONF_SECTION_AQBANKING, KEY_VERBOSE_DEBUG, NULL))
+    {
+        GWEN_Logger_SetLevel(NULL, GWEN_LoggerLevel_Info);
+        GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Info);
+        GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Debug);
+    }
+    else
+    {
+        GWEN_Logger_SetLevel(NULL, GWEN_LoggerLevel_Error);
+        GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Error);
+        GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Warning);
+    }
     gnc_GWEN_Gui_log_init();
 }
 
@@ -95,7 +111,7 @@ gnc_GWEN_Fini(void)
 {
     /* Shutdown the GWEN_GUIs */
     gnc_GWEN_Gui_shutdown();
-    GWEN_Logger_SetLevel(NULL, GWEN_LoggerLevel_Warning);
+    GWEN_Logger_SetLevel(NULL, GWEN_LoggerLevel_Error);
     GWEN_Logger_SetLevel(GWEN_LOGDOMAIN, GWEN_LoggerLevel_Warning);
     GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevel_Warning);
 
@@ -103,12 +119,15 @@ gnc_GWEN_Fini(void)
     GWEN_Fini();
 }
 
+static GWEN_GUI *gnc_gwengui_extended_by_ABBanking;
+
 AB_BANKING *
 gnc_AB_BANKING_new(void)
 {
     AB_BANKING *api;
 
-    if (gnc_AB_BANKING) {
+    if (gnc_AB_BANKING)
+    {
         /* API cached. */
         api = gnc_AB_BANKING;
 
@@ -116,13 +135,63 @@ gnc_AB_BANKING_new(void)
         if (gnc_AB_BANKING_refcount == 0)
             g_return_val_if_fail(AB_Banking_Init(api) == 0, NULL);
 
-    } else {
+    }
+    else
+    {
         api = AB_Banking_new("gnucash", NULL, 0);
         g_return_val_if_fail(api, NULL);
 
+#ifdef AQBANKING_VERSION_4_PLUS
+        /* Check for config migration */
+        if (AB_Banking_HasConf4(api
+# ifndef AQBANKING_VERSION_5_PLUS
+                                , 0
+# endif
+                               ) != 0)
+        {
+            if (AB_Banking_HasConf3(api
+# ifndef AQBANKING_VERSION_5_PLUS
+                                    , 0
+# endif
+                                   ) == 0)
+            {
+                g_message("gnc_AB_BANKING_new: importing aqbanking3 configuration\n");
+                if (AB_Banking_ImportConf3(api
+# ifndef AQBANKING_VERSION_5_PLUS
+                                           , 0
+# endif
+                                          ) < 0)
+                {
+                    g_message("gnc_AB_BANKING_new: unable to import aqbanking3 configuration\n");
+                }
+            }
+            else if (AB_Banking_HasConf2(api
+# ifndef AQBANKING_VERSION_5_PLUS
+                                         , 0
+# endif
+                                        ) == 0)
+            {
+                g_message("gnc_AB_BANKING_new: importing aqbanking2 configuration\n");
+                if (AB_Banking_ImportConf2(api
+# ifndef AQBANKING_VERSION_5_PLUS
+                                           , 0
+# endif
+                                          ) < 0)
+                {
+                    g_message("gnc_AB_BANKING_new: unable to import aqbanking2 configuration\n");
+                }
+            }
+        }
+#endif /* AQBANKING_VERSION_4_PLUS */
+
         /* Init the API */
         g_return_val_if_fail(AB_Banking_Init(api) == 0, NULL);
 
+#ifdef AQBANKING_VERSION_5_PLUS
+        gnc_gwengui_extended_by_ABBanking = GWEN_Gui_GetGui();
+        AB_Gui_Extend(gnc_gwengui_extended_by_ABBanking, api);
+#endif /* AQBANKING_VERSION_5_PLUS */
+
         /* Cache it */
         gnc_AB_BANKING = api;
         gnc_AB_BANKING_refcount = 0;
@@ -139,11 +208,12 @@ gnc_AB_BANKING_delete(AB_BANKING *api)
     if (!api)
         api = gnc_AB_BANKING;
 
-    if (api) {
-        if (api == gnc_AB_BANKING) {
+    if (api)
+    {
+        if (api == gnc_AB_BANKING)
+        {
             gnc_AB_BANKING = NULL;
-            if (gnc_AB_BANKING_refcount > 0)
-                AB_Banking_Fini(api);
+            gnc_AB_BANKING_fini(api);
         }
 
         AB_Banking_free(api);
@@ -154,10 +224,25 @@ gnc_AB_BANKING_delete(AB_BANKING *api)
 gint
 gnc_AB_BANKING_fini(AB_BANKING *api)
 {
-    if (api == gnc_AB_BANKING) {
+    if (api == gnc_AB_BANKING)
+    {
         if (--gnc_AB_BANKING_refcount == 0)
+        {
+#ifdef AQBANKING_VERSION_5_PLUS
+            if (gnc_gwengui_extended_by_ABBanking)
+                AB_Gui_Unextend(gnc_gwengui_extended_by_ABBanking);
+            gnc_gwengui_extended_by_ABBanking = NULL;
+#endif /* AQBANKING_VERSION_5_PLUS */
             return AB_Banking_Fini(api);
-    } else {
+        }
+    }
+    else
+    {
+#ifdef AQBANKING_VERSION_5_PLUS
+        if (gnc_gwengui_extended_by_ABBanking)
+            AB_Gui_Unextend(gnc_gwengui_extended_by_ABBanking);
+        gnc_gwengui_extended_by_ABBanking = NULL;
+#endif /* AQBANKING_VERSION_5_PLUS */
         return AB_Banking_Fini(api);
     }
     return 0;
@@ -175,20 +260,24 @@ gnc_ab_get_ab_account(const AB_BANKING *api, Account *gnc_acc)
     accountid = gnc_ab_get_account_accountid(gnc_acc);
     account_uid = gnc_ab_get_account_uid (gnc_acc);
 
-    if (account_uid > 0) {
+    if (account_uid > 0)
+    {
         ab_account = AB_Banking_GetAccount(api, account_uid);
 
-        if (!ab_account && bankcode && *bankcode && accountid && *accountid) {
+        if (!ab_account && bankcode && *bankcode && accountid && *accountid)
+        {
             g_message("gnc_ab_get_ab_account: No AB_ACCOUNT found for UID %d, "
                       "trying bank code\n", account_uid);
             ab_account = AB_Banking_GetAccountByCodeAndNumber(api, bankcode,
-                                                              accountid);
+                         accountid);
         }
         return ab_account;
 
-    } else if (bankcode && *bankcode && accountid && *accountid) {
+    }
+    else if (bankcode && *bankcode && accountid && *accountid)
+    {
         ab_account = AB_Banking_GetAccountByCodeAndNumber(api, bankcode,
-                                                          accountid);
+                     accountid);
         return ab_account;
     }
 
@@ -223,12 +312,15 @@ join_ab_strings_cb(const gchar *str, gpointer user_data)
     g_strstrip(tmp);
     gnc_utf8_strip_invalid(tmp);
 
-    if (*acc) {
+    if (*acc)
+    {
         gchar *join = g_strjoin(" ", *acc, tmp, (gchar*) NULL);
         g_free(*acc);
         g_free(tmp);
         *acc = join;
-    } else {
+    }
+    else
+    {
         *acc = tmp;
     }
     return NULL;
@@ -247,7 +339,8 @@ gnc_ab_get_remote_name(const AB_TRANSACTION *ab_trans)
         GWEN_StringList_ForEach(ab_remote_name, join_ab_strings_cb,
                                 &gnc_other_name);
 
-    if (!gnc_other_name || !*gnc_other_name) {
+    if (!gnc_other_name || !*gnc_other_name)
+    {
         g_free(gnc_other_name);
         gnc_other_name = NULL;
     }
@@ -277,21 +370,30 @@ gnc_ab_get_purpose(const AB_TRANSACTION *ab_trans)
 gchar *
 gnc_ab_description_to_gnc(const AB_TRANSACTION *ab_trans)
 {
-  /* Description */
+    /* Description */
     gchar *description = gnc_ab_get_purpose(ab_trans);
     gchar *other_name = gnc_ab_get_remote_name(ab_trans);
     gchar *retval;
 
-    if (other_name) {
-        if (description && *description) {
+    if (other_name)
+    {
+        if (description && *description)
+        {
             retval = g_strdup_printf("%s; %s", description, other_name);
-        } else {
+        }
+        else
+        {
             retval = g_strdup(other_name);
         }
-    } else {
-        if (description && *description) {
+    }
+    else
+    {
+        if (description && *description)
+        {
             retval = g_strdup(description);
-        } else {
+        }
+        else
+        {
             retval = g_strdup(_("Unspecified"));
         }
     }
@@ -322,11 +424,14 @@ gnc_ab_memo_to_gnc(const AB_TRANSACTION *ab_trans)
     gnc_utf8_strip_invalid(ab_other_accountid);
     gnc_utf8_strip_invalid(ab_other_bankcode);
 
-    if (ab_other_accountid && *ab_other_accountid) {
+    if (ab_other_accountid && *ab_other_accountid)
+    {
         retval = g_strdup_printf("%s %s %s %s",
                                  _("Account"), ab_other_accountid,
                                  _("Bank"), ab_other_bankcode);
-    } else {
+    }
+    else
+    {
         retval = g_strdup("");
     }
     g_free(ab_other_accountid);
@@ -346,8 +451,6 @@ gnc_ab_trans_to_gnc(const AB_TRANSACTION *ab_trans, Account *gnc_acc)
     const char *custref;
     gchar *description;
     Split *split;
-    const AB_VALUE *ab_value;
-    gnc_numeric gnc_amount;
     gchar *memo;
 
     g_return_val_if_fail(ab_trans && gnc_acc, NULL);
@@ -357,20 +460,16 @@ gnc_ab_trans_to_gnc(const AB_TRANSACTION *ab_trans, Account *gnc_acc)
     gnc_trans = xaccMallocTransaction(book);
     xaccTransBeginEdit(gnc_trans);
 
-    /* Set OFX unique transaction ID */
-    fitid = AB_Transaction_GetFiId(ab_trans);
-    if (fitid && *fitid)
-        gnc_import_set_trans_online_id(gnc_trans, fitid);
-
     /* Date / Time */
     valuta_date = AB_Transaction_GetValutaDate(ab_trans);
-    if (!valuta_date) {
+    if (!valuta_date)
+    {
         const GWEN_TIME *normal_date = AB_Transaction_GetDate(ab_trans);
         if (normal_date)
             valuta_date = normal_date;
     }
     if (valuta_date)
-        xaccTransSetDateSecs(gnc_trans, GWEN_Time_toTime_t(valuta_date));
+        xaccTransSetDatePostedSecs(gnc_trans, GWEN_Time_toTime_t(valuta_date));
     else
         g_warning("transaction_cb: Oops, date 'valuta_date' was NULL");
 
@@ -383,7 +482,7 @@ gnc_ab_trans_to_gnc(const AB_TRANSACTION *ab_trans, Account *gnc_acc)
     /* Number.  We use the "customer reference", if there is one. */
     custref = AB_Transaction_GetCustomerReference(ab_trans);
     if (custref && *custref
-        && g_ascii_strncasecmp(custref, "NONREF", 6) != 0)
+            && g_ascii_strncasecmp(custref, "NONREF", 6) != 0)
         xaccTransSetNum(gnc_trans, custref);
 
     /* Description */
@@ -400,15 +499,33 @@ gnc_ab_trans_to_gnc(const AB_TRANSACTION *ab_trans, Account *gnc_acc)
     xaccSplitSetParent(split, gnc_trans);
     xaccSplitSetAccount(split, gnc_acc);
 
-    /* Amount into the split */
-    ab_value = AB_Transaction_GetValue(ab_trans);
-    gnc_amount = double_to_gnc_numeric(
-        ab_value ? AB_Value_GetValueAsDouble(ab_value) : 0.0,
-        xaccAccountGetCommoditySCU(gnc_acc),
-        GNC_RND_ROUND);
-    if (!ab_value)
-        g_warning("transaction_cb: Oops, value was NULL.  Using 0");
-    xaccSplitSetBaseValue(split, gnc_amount, xaccAccountGetCommodity(gnc_acc));
+    /* Set OFX unique transaction ID */
+    fitid = AB_Transaction_GetFiId(ab_trans);
+    if (fitid && *fitid)
+        gnc_import_set_split_online_id(split, fitid);
+
+    {
+        /* Amount into the split */
+        const AB_VALUE *ab_value = AB_Transaction_GetValue(ab_trans);
+        double d_value = ab_value ? AB_Value_GetValueAsDouble (ab_value) : 0.0;
+        AB_TRANSACTION_TYPE ab_type = AB_Transaction_GetType (ab_trans);
+        gnc_numeric gnc_amount;
+
+        /*printf("Transaction with value %f has type %d\n", d_value, ab_type);*/
+        /* If the value is positive, but the transaction type says the
+           money is transferred away from our account (Transfer instead of
+           DebitNote), we switch the value to negative. */
+        if (d_value > 0.0 && ab_type == AB_Transaction_TypeTransfer)
+            d_value = -d_value;
+
+        gnc_amount = double_to_gnc_numeric(
+                         d_value,
+                         xaccAccountGetCommoditySCU(gnc_acc),
+                         GNC_HOW_RND_ROUND_HALF_UP);
+        if (!ab_value)
+            g_warning("transaction_cb: Oops, value was NULL.  Using 0");
+        xaccSplitSetBaseValue(split, gnc_amount, xaccAccountGetCommodity(gnc_acc));
+    }
 
     /* Memo in the Split. */
     memo = gnc_ab_memo_to_gnc(ab_trans);
@@ -440,9 +557,10 @@ gnc_ab_accinfo_to_gnc_acc(AB_IMEXPORTER_ACCOUNTINFO *acc_info)
                             accountnumber ? accountnumber : "",
                             (gchar*)NULL);
     gnc_acc = gnc_import_select_account(
-        NULL, online_id, 1, AB_ImExporterAccountInfo_GetAccountName(acc_info),
-        NULL, ACCT_TYPE_NONE, NULL, NULL);
-    if (!gnc_acc) {
+                  NULL, online_id, 1, AB_ImExporterAccountInfo_GetAccountName(acc_info),
+                  NULL, ACCT_TYPE_NONE, NULL, NULL);
+    if (!gnc_acc)
+    {
         g_warning("gnc_ab_accinfo_to_gnc_acc: Could not determine source account"
                   " for online_id %s", online_id);
     }
@@ -456,16 +574,15 @@ txn_transaction_cb(const AB_TRANSACTION *element, gpointer user_data)
 {
     GncABImExContextImport *data = user_data;
     Transaction *gnc_trans;
+    GncABTransType trans_type;
 
     g_return_val_if_fail(element && data, NULL);
 
     /* Create a GnuCash transaction from ab_trans */
     gnc_trans = gnc_ab_trans_to_gnc(element, data->gnc_acc);
 
-    /* Instead of xaccTransCommitEdit(gnc_trans)  */
-    gnc_gen_trans_list_add_trans(data->generic_importer, gnc_trans);
-
-    if (data->execute_txns && data->ab_acc) {
+    if (data->execute_txns && data->ab_acc)
+    {
         AB_TRANSACTION *ab_trans = AB_Transaction_dup(element);
         AB_JOB *job;
 
@@ -477,34 +594,104 @@ txn_transaction_cb(const AB_TRANSACTION *element, gpointer user_data)
             ab_trans, AB_Account_GetAccountNumber(data->ab_acc));
         AB_Transaction_SetLocalCountry(ab_trans, "DE");
 
-        job = gnc_ab_get_trans_job(data->ab_acc, ab_trans, SINGLE_DEBITNOTE);
+
+        switch (AB_Transaction_GetType(ab_trans))
+        {
+        case AB_Transaction_TypeDebitNote:
+            trans_type = SINGLE_DEBITNOTE;
+            break;
+        case AB_Transaction_TypeTransaction:
+            /* trans_type = SINGLE_INTERNAL_TRANSFER;
+             * break; */
+        case AB_Transaction_TypeEuTransfer:
+        case AB_Transaction_TypeTransfer:
+        default:
+            trans_type = SINGLE_TRANSFER;
+        } /* switch */
+
+        job = gnc_ab_get_trans_job(data->ab_acc, ab_trans, trans_type);
 
         /* Check whether we really got a job */
-        if (!job) {
+        if (!job || AB_Job_CheckAvailability(job
+#ifndef AQBANKING_VERSION_5_PLUS
+                                             , 0
+#endif
+                                            ))
+        {
             /* Oops, no job, probably not supported by bank */
             if (gnc_verify_dialog(
-                    NULL, FALSE, "%s",
-                    _("The backend found an error during the preparation "
-                      "of the job. It is not possible to execute this job. \n"
-                      "\n"
-                      "Most probable the bank does not support your chosen "
-                      "job or your Online Banking account does not have the permission "
-                      "to execute this job. More error messages might be "
-                      "visible on your console log.\n"
-                      "\n"
-                      "Do you want to enter the job again?"))) {
-                gnc_error_dialog(NULL, "Sorry, not implemented yet.");
+                        NULL, FALSE, "%s",
+                        _("The backend found an error during the preparation "
+                          "of the job. It is not possible to execute this job. \n"
+                          "\n"
+                          "Most probable the bank does not support your chosen "
+                          "job or your Online Banking account does not have the permission "
+                          "to execute this job. More error messages might be "
+                          "visible on your console log.\n"
+                          "\n"
+                          "Do you want to enter the job again?")))
+            {
+                gnc_error_dialog(NULL, "Sorry, not implemented yet. Please check the console or trace file logs to see which job was rejected.");
             }
-            /* else */
         }
-        AB_Job_List2_PushBack(data->job_list, job);
+        else
+        {
+            gnc_gen_trans_list_add_trans_with_ref_id(data->generic_importer, gnc_trans, AB_Job_GetJobId(job));
 
+            /* AB_Job_List2_PushBack(data->job_list, job); -> delayed until trans is successfully imported */
+            g_datalist_set_data(&data->tmp_job_list, gnc_AB_JOB_to_readable_string(job), job);
+        }
         AB_Transaction_free(ab_trans);
     }
+    else
+    {
+        /* Instead of xaccTransCommitEdit(gnc_trans)  */
+        gnc_gen_trans_list_add_trans(data->generic_importer, gnc_trans);
+    }
 
     return NULL;
 }
 
+static void gnc_ab_trans_processed_cb(GNCImportTransInfo *trans_info,
+                                      gboolean imported,
+                                      gpointer user_data)
+{
+    GncABImExContextImport *data = user_data;
+    gchar *jobname = gnc_AB_JOB_ID_to_string(gnc_import_TransInfo_get_ref_id(trans_info));
+    AB_JOB *job = g_datalist_get_data(&data->tmp_job_list, jobname);
+
+    if (imported)
+    {
+        AB_Job_List2_PushBack(data->job_list, job);
+    }
+    else
+    {
+        AB_Job_free(job);
+    }
+
+    g_datalist_remove_data(&data->tmp_job_list, jobname);
+}
+
+gchar *
+gnc_AB_JOB_to_readable_string(const AB_JOB *job)
+{
+    if (job)
+    {
+        return gnc_AB_JOB_ID_to_string(AB_Job_GetJobId(job));
+    }
+    else
+    {
+        return gnc_AB_JOB_ID_to_string(0);
+    }
+}
+gchar *
+gnc_AB_JOB_ID_to_string(gulong job_id)
+{
+    return g_strdup_printf("job_%lu", job_id);
+}
+
+
+
 static AB_IMEXPORTER_ACCOUNTINFO *
 txn_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
 {
@@ -523,14 +710,18 @@ txn_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
     else
         data->awaiting |= FOUND_TRANSACTIONS;
 
-    if (!(data->awaiting & AWAIT_TRANSACTIONS)) {
+    if (!(data->awaiting & AWAIT_TRANSACTIONS))
+    {
         if (gnc_verify_dialog(data->parent, TRUE, "%s",
                               _("The bank has sent transaction information "
                                 "in its response."
                                 "\n"
-                                "Do you want to import it?"))) {
+                                "Do you want to import it?")))
+        {
             data->awaiting |= AWAIT_TRANSACTIONS;
-        } else {
+        }
+        else
+        {
             data->awaiting |= IGNORE_TRANSACTIONS;
             return NULL;
         }
@@ -541,27 +732,38 @@ txn_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
     if (!gnc_acc) return NULL;
     data->gnc_acc = gnc_acc;
 
-    if (data->execute_txns) {
+    if (data->execute_txns)
+    {
         /* Retrieve the aqbanking account that belongs to this gnucash
          * account */
         data->ab_acc = gnc_ab_get_ab_account(data->api, gnc_acc);
-        if (!data->ab_acc) {
+        if (!data->ab_acc)
+        {
             gnc_error_dialog(NULL, "%s",
                              _("No Online Banking account found for this "
                                "gnucash account. These transactions will "
                                "not be executed by Online Banking."));
         }
-    } else {
+    }
+    else
+    {
         data->ab_acc = NULL;
     }
 
     if (!data->generic_importer)
+    {
         data->generic_importer = gnc_gen_trans_list_new(data->parent, NULL,
-                                                        TRUE, 14);
+                                 TRUE, 14);
+        if (data->execute_txns)
+        {
+            gnc_gen_trans_list_add_tp_cb(data->generic_importer,
+                                         gnc_ab_trans_processed_cb, data);
+        }
+    }
 
     /* Iterate through all transactions */
     AB_ImExporterAccountInfo_TransactionsForEach(element, txn_transaction_cb,
-                                                 data);
+            data);
 
     return NULL;
 }
@@ -595,9 +797,11 @@ bal_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
 
     /* Lookup the most recent ACCOUNT_STATUS available */
     item = AB_ImExporterAccountInfo_GetFirstAccountStatus(element);
-    while (item) {
+    while (item)
+    {
         const GWEN_TIME *item_time = AB_AccountStatus_GetTime(item);
-        if (!best || GWEN_Time_Diff(best_time, item_time) < 0.0) {
+        if (!best || GWEN_Time_Diff(best_time, item_time) < 0.0)
+        {
             best = item;
             best_time = item_time;
         }
@@ -605,7 +809,8 @@ bal_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
     }
 
     booked_bal = AB_AccountStatus_GetBookedBalance(best);
-    if (!(data->awaiting & AWAIT_BALANCES)) {
+    if (!(data->awaiting & AWAIT_BALANCES))
+    {
         /* Ignore zero balances if we don't await a balance */
         if (!booked_bal || AB_Value_IsZero(AB_Balance_GetValue(booked_bal)))
             return NULL;
@@ -615,9 +820,12 @@ bal_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
                               _("The bank has sent balance information "
                                 "in its response."
                                 "\n"
-                                "Do you want to import it?"))) {
+                                "Do you want to import it?")))
+        {
             data->awaiting |= AWAIT_BALANCES;
-        } else {
+        }
+        else
+        {
             data->awaiting |= IGNORE_BALANCES;
             return NULL;
         }
@@ -629,23 +837,32 @@ bal_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
     data->gnc_acc = gnc_acc;
 
     /* Lookup booked balance and time */
-    if (booked_bal) {
+    if (booked_bal)
+    {
         const GWEN_TIME *ti = AB_Balance_GetTime(booked_bal);
-        if (ti) {
+        if (ti)
+        {
             booked_tt =  GWEN_Time_toTime_t(ti);
-        } else {
+        }
+        else
+        {
             /* No time found? Use today because the HBCI query asked for today's
              * balance. */
             booked_tt = gnc_timet_get_day_start(time(NULL));
         }
         booked_val = AB_Balance_GetValue(booked_bal);
-        if (booked_val) {
+        if (booked_val)
+        {
             booked_value = AB_Value_GetValueAsDouble(booked_val);
-        } else {
+        }
+        else
+        {
             g_warning("bal_accountinfo_cb: booked_val == NULL.  Assuming 0");
             booked_value = 0.0;
         }
-    } else {
+    }
+    else
+    {
         g_warning("bal_accountinfo_cb: booked_bal == NULL.  Assuming 0");
         booked_tt = 0;
         booked_value = 0.0;
@@ -653,54 +870,61 @@ bal_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
 
     /* Lookup noted balance */
     noted_bal = AB_AccountStatus_GetNotedBalance(best);
-    if (noted_bal) {
+    if (noted_bal)
+    {
         noted_val = AB_Balance_GetValue(noted_bal);
         if (noted_val)
             noted_value = AB_Value_GetValueAsDouble(noted_val);
-        else {
+        else
+        {
             g_warning("bal_accountinfo_cb: noted_val == NULL.  Assuming 0");
             noted_value = 0.0;
         }
-    } else {
+    }
+    else
+    {
         g_warning("bal_accountinfo_cb: noted_bal == NULL.  Assuming 0");
         noted_value = 0.0;
     }
 
     value = double_to_gnc_numeric(booked_value,
                                   xaccAccountGetCommoditySCU(gnc_acc),
-                                  GNC_RND_ROUND);
-    if (noted_value == 0.0 && booked_value == 0.0) {
+                                  GNC_HOW_RND_ROUND_HALF_UP);
+    if (noted_value == 0.0 && booked_value == 0.0)
+    {
         dialog = gtk_message_dialog_new(
-            GTK_WINDOW(data->parent),
-            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-            GTK_MESSAGE_INFO,
-            GTK_BUTTONS_OK,
-            "%s",
-            /* Translators: Strings from this file are needed only in
-             * countries that have one of aqbanking's Online Banking
-             * techniques available. This is 'OFX DirectConnect'
-             * (U.S. and others), 'HBCI' (in Germany), or 'YellowNet'
-             * (Switzerland). If none of these techniques are available
-             * in your country, you may safely ignore strings from the
-             * import-export/hbci subdirectory. */
-            _("The downloaded Online Banking Balance was zero.\n\n"
-              "Either this is the correct balance, or your bank does not "
-              "support Balance download in this Online Banking version. "
-              "In the latter case you should choose a different "
-              "Online Banking version number in the Online Banking "
-              "(AqBanking or HBCI) Setup. After that, try again to "
-              "download the Online Banking Balance."));
+                     GTK_WINDOW(data->parent),
+                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                     GTK_MESSAGE_INFO,
+                     GTK_BUTTONS_OK,
+                     "%s",
+                     /* Translators: Strings from this file are needed only in
+                      * countries that have one of aqbanking's Online Banking
+                      * techniques available. This is 'OFX DirectConnect'
+                      * (U.S. and others), 'HBCI' (in Germany), or 'YellowNet'
+                      * (Switzerland). If none of these techniques are available
+                      * in your country, you may safely ignore strings from the
+                      * import-export/hbci subdirectory. */
+                     _("The downloaded Online Banking Balance was zero.\n\n"
+                       "Either this is the correct balance, or your bank does not "
+                       "support Balance download in this Online Banking version. "
+                       "In the latter case you should choose a different "
+                       "Online Banking version number in the Online Banking "
+                       "(AqBanking or HBCI) Setup. After that, try again to "
+                       "download the Online Banking Balance."));
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
 
-    } else {
+    }
+    else
+    {
         gnc_numeric reconc_balance = xaccAccountGetReconciledBalance(gnc_acc);
 
         gchar *booked_str = gnc_AB_VALUE_to_readable_string(booked_val);
         gchar *message1 = g_strdup_printf(
-            _("Result of Online Banking job: \n"
-              "Account booked balance is %s"),
-            booked_str);
+                              _("Result of Online Banking job: \n"
+                                "Account booked balance is %s"),
+                              booked_str);
         gchar *message2 =
             (noted_value == 0.0) ?
             g_strdup("") :
@@ -708,24 +932,27 @@ bal_accountinfo_cb(AB_IMEXPORTER_ACCOUNTINFO *element, gpointer user_data)
                               "has a noted balance of %s\n"),
                             gnc_AB_VALUE_to_readable_string(noted_val));
 
-        if (gnc_numeric_equal(value, reconc_balance)) {
+        if (gnc_numeric_equal(value, reconc_balance))
+        {
             const gchar *message3 =
                 _("The booked balance is identical to the current "
                   "reconciled balance of the account.");
             dialog = gtk_message_dialog_new(
-                GTK_WINDOW(data->parent),
-                GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                GTK_MESSAGE_INFO,
-                GTK_BUTTONS_OK,
-                "%s\n%s\n%s",
-                message1, message2, message3);
+                         GTK_WINDOW(data->parent),
+                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                         GTK_MESSAGE_INFO,
+                         GTK_BUTTONS_OK,
+                         "%s\n%s\n%s",
+                         message1, message2, message3);
             gtk_dialog_run(GTK_DIALOG(dialog));
             gtk_widget_destroy(GTK_WIDGET(dialog));
 
-        } else {
+        }
+        else
+        {
             const char *message3 = _("Reconcile account now?");
 
-            show_recn_window = gnc_verify_dialog(data->parent,TRUE, "%s\n%s\n%s",
+            show_recn_window = gnc_verify_dialog(data->parent, TRUE, "%s\n%s\n%s",
                                                  message1, message2, message3);
         }
         g_free(booked_str);
@@ -765,9 +992,12 @@ gnc_ab_import_context(AB_IMEXPORTER_CONTEXT *context,
     data->execute_txns = execute_txns;
     data->api = api;
     data->parent = parent;
-    data->job_list = NULL;
+    data->job_list = AB_Job_List2_new();
+    data->tmp_job_list = NULL;
     data->generic_importer = NULL;
 
+    g_datalist_init(&data->tmp_job_list);
+
     /* Import transactions */
     if (!(awaiting & IGNORE_TRANSACTIONS))
         AB_ImExporterContext_AccountInfoForEach(context, txn_accountinfo_cb,
@@ -804,3 +1034,28 @@ gnc_ab_ieci_run_matcher(GncABImExContextImport *ieci)
 
     return gnc_gen_trans_list_run(ieci->generic_importer);
 }
+
+GWEN_DB_NODE *
+gnc_ab_get_permanent_certs(void)
+{
+    int rv;
+    GWEN_DB_NODE *perm_certs = NULL;
+    AB_BANKING *banking = gnc_AB_BANKING_new();
+
+    g_return_val_if_fail(banking, NULL);
+#ifdef AQBANKING_VERSION_4_PLUS
+    rv = AB_Banking_LoadSharedConfig(banking, "certs", &perm_certs
+# ifndef AQBANKING_VERSION_5_PLUS
+                                     , 0
+# endif
+                                    );
+#else
+    /* FIXME: Add code for older AqBanking versions */
+    /* See QBankmanager 0.9.50 in src/kbanking/libs/kbanking.cpp lines 323ff
+       for a proper example of how to do this */
+    rv = 0;
+#endif
+    gnc_AB_BANKING_fini(banking);
+    g_return_val_if_fail(rv >= 0, NULL);
+    return perm_certs;
+}
