@@ -15,75 +15,82 @@
  * along with this program; if not, contact:                        *
  *                                                                  *
  * Free Software Foundation           Voice:  +1-617-542-5942       *
- * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
- * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
  *                                                                  *
 \********************************************************************/
-
-/*
- * FILE:
- * table-allgui.h
+/** @addtogroup GUI
+ *  @{
+ */
+/** @addtogroup Register Registers, Ledgers and Journals
+ *  @{
+ */
+/** @addtogroup RegisterCore Register Core
+ *  @{
+ */
+/** @file table-allgui.h
+ *  @brief Declarations for the Table object
+ *  @author Copyright (c) 1998,1999,2000 Linas Vepstas
+ *  @author Copyright (c) 2000-2001 Dave Peticolas
  *
- * FUNCTION:
- * The Table object defines the structure and the GUI required
- * to display a two-dimensional grid. It provides several 
- * important functions:
- * -- An array of virtual cells. These cells contain:
- *    -- the cellblock handler for that virtual cell.
- *    -- a user data pointer
- * -- Tab-traversing mechanism so that operator can tab in a
- *    predefined order between cells.
+ *  @details
+ *  The Table object defines the structure and the GUI required
+ *  to display a two-dimensional grid. It provides several
+ *  important functions:
+ *  - An array of virtual cells. These cells contain:
+ *     - the cellblock handler for that virtual cell.
+ *     - a user data pointer
+ *  - Tab-traversing mechanism so that operator can tab in a
+ *     predefined order between cells.
  *
- * Please see src/doc/design/gnucash-design.info for additional information.
+ *  Please see src/doc/design/gnucash-design.info for additional information.
  *
- * This implements the gui-independent parts of the table 
- * infrastructure. Additional, GUI-dependent parts are implemented
- * in table-gnome.c.
+ *  This implements the gui-independent parts of the table
+ *  infrastructure. Additional, GUI-dependent parts are implemented
+ *  in table-gnome.c.
  *
- * CONCEPTS:
- * The following apply to the rows in a table:
- * -- a phys row can belong to only one virt row at a time.
- * -- a cursor is always the same size as the virt row its on,
- * -- there is only one cursor for a given virt row.
- * -- there is no overlap; a phys row can only belong to one virt row.
+ *  The following concepts apply to the rows in a table:
+ *  - a cursor is always the same size as the virt row its on,
+ *  - there is only one cursor for a given virt row.
+ *  - there is no overlap; a phys row can only belong to one virt row.
  *
- * Lets say there are three cursors T(rans),S(plit), and B(lank).  
- * Lets say that these are used to 'print' the following table layout:
- * 
- *       virt row 1   T
- *       virt row 2   S
- *       virt row 3   B
- *       virt row 4   T
- *       virt row 5   S
- *       virt row 6   S
- *       virt row 7   S
- *       virt row 8   S
- *       virt row 9   B
+ *  Lets say there are three cursors T(rans), S(plit), and B(lank).
+ *  Lets say that these are used to 'print' the following table layout:
  *
- * You can see there is only one cursor per virt row.  There is no overlap
- * between cursors and virt rows, the correspondence is one to one.  Note
- * that the three cursors T,S and B may consist of one, or more phys rows,
- * e.g. B and S may be one line each, but T may be two lines.  Thus, we 
- * have the following physical layout:
+@verbatim
+virtual row 1   T
+virtual row 2   S
+virtual row 3   B
+virtual row 4   T
+virtual row 5   S
+virtual row 6   S
+virtual row 7   S
+virtual row 8   S
+virtual row 9   B
+@endverbatim
  *
- *      phys row 1    virt row 1   T
- *      phys row 2    virt row 1   T
- *      phys row 3    virt row 2   S
- *      phys row 4    virt row 3   B
- *      phys row 5    virt row 4   T
- *      phys row 6    virt row 4   T
- *      phys row 7    virt row 5   S
- *      phys row 8    virt row 6   S
- *      phys row 9    virt row 7   S
- *      phys row 10   virt row 8   S
- *      phys row 11   virt row 9   B
+ *  You can see there is only one cursor per virtual row. There is no overlap
+ *  between cursors and virtual rows; the correspondence is one to one. Note
+ *  that the three cursors T, S and B may consist of one, or more physical rows,
+ *  e.g. B and S may be one line each, but T may be two lines. Thus, we
+ *  have the following physical layout:
  *
- * This layout remains static until the next time that the table is 
- * re-'printed'.
+@verbatim
+physical row 1    virtual row 1   T
+physical row 2    virtual row 1   T
+physical row 3    virtual row 2   S
+physical row 4    virtual row 3   B
+physical row 5    virtual row 4   T
+physical row 6    virtual row 4   T
+physical row 7    virtual row 5   S
+physical row 8    virtual row 6   S
+physical row 9    virtual row 7   S
+physical row 10   virtual row 8   S
+physical row 11   virtual row 9   B
+@endverbatim
  *
- * HISTORY:
- * Copyright (c) 1998,1999,2000 Linas Vepstas
- * Copyright (c) 2000-2001 Dave Peticolas
+ *  This layout remains static until the next time that the table is
+ *  re-'printed'.
  */
 
 #ifndef TABLE_ALLGUI_H
@@ -99,16 +106,15 @@
 #include "table-layout.h"
 #include "table-model.h"
 
-
 /* The VirtualCell structure holds information about each virtual cell. */
 typedef struct
 {
-  CellBlock *cellblock;  /* Array of physical cells */
-  gpointer   vcell_data; /* Used by higher-level code */
+    CellBlock *cellblock;  /* Array of physical cells */
+    gpointer   vcell_data; /* Used by higher-level code */
 
-  /* flags */
-  unsigned int visible : 1;             /* visible in the GUI */
-  unsigned int start_primary_color : 1; /* color usage flag */
+    /* flags */
+    unsigned int visible : 1;             /* visible in the GUI */
+    unsigned int start_primary_color : 1; /* color usage flag */
 } VirtualCell;
 
 typedef struct table Table;
@@ -122,32 +128,32 @@ typedef void (*TableDestroyCB) (Table *table);
 
 typedef struct
 {
-  TableCursorRefreshCB cursor_refresh;
+    TableCursorRefreshCB cursor_refresh;
 
-  TableRedrawHelpCB redraw_help;
-  TableDestroyCB destroy;
+    TableRedrawHelpCB redraw_help;
+    TableDestroyCB destroy;
 } TableGUIHandlers;
 
 struct table
 {
-  TableLayout  *layout;
-  TableModel   *model;
-  TableControl *control;
+    TableLayout  *layout;
+    TableModel   *model;
+    TableControl *control;
 
-  int num_virt_rows;
-  int num_virt_cols;
+    int num_virt_rows;
+    int num_virt_cols;
 
-  CellBlock *current_cursor;
+    CellBlock *current_cursor;
 
-  VirtualLocation current_cursor_loc;
+    VirtualLocation current_cursor_loc;
 
-  /* private data */
+    /* private data */
 
-  /* The virtual cell table */
-  GTable *virt_cells;
+    /* The virtual cell table */
+    GTable *virt_cells;
 
-  TableGUIHandlers gui_handlers;
-  gpointer ui_data;
+    TableGUIHandlers gui_handlers;
+    gpointer ui_data;
 };
 
 
@@ -158,6 +164,7 @@ void gnc_table_set_default_gui_handlers (TableGUIHandlers *gui_handlers);
 Table *     gnc_table_new (TableLayout *layout,
                            TableModel *model,
                            TableControl *control);
+void        gnc_virtual_location_init (VirtualLocation *vloc);
 
 void        gnc_table_save_state (Table *table);
 void        gnc_table_destroy (Table *table);
@@ -165,35 +172,35 @@ void        gnc_table_destroy (Table *table);
 
 /* Functions to work with current cursor */
 int         gnc_table_current_cursor_changed (Table *table,
-                                              gboolean include_conditional);
+        gboolean include_conditional);
 
 void        gnc_table_clear_current_cursor_changes (Table *table);
 
 void        gnc_table_save_current_cursor (Table *table, CursorBuffer *buffer);
 void        gnc_table_restore_current_cursor (Table *table,
-                                              CursorBuffer *buffer);
+        CursorBuffer *buffer);
 
 const char * gnc_table_get_current_cell_name (Table *table);
 
 gboolean    gnc_table_get_current_cell_location (Table *table,
-                                                 const char *cell_name,
-                                                 VirtualLocation *virt_loc);
+        const char *cell_name,
+        VirtualLocation *virt_loc);
 
 
 /* This function checks the given location and returns true
  * if it is out of bounds of the table. */
 gboolean gnc_table_virtual_cell_out_of_bounds (Table *table,
-                                               VirtualCellLocation vcell_loc);
+        VirtualCellLocation vcell_loc);
 
 gboolean gnc_table_virtual_location_in_header (Table *table,
-                                               VirtualLocation virt_loc);
+        VirtualLocation virt_loc);
 
 
 /* This function returns the virtual cell associated with a particular
  *   virtual location. If the location is out of bounds, NULL is *
  *   returned. */
 VirtualCell *  gnc_table_get_virtual_cell (Table *table,
-                                           VirtualCellLocation vcell_loc);
+        VirtualCellLocation vcell_loc);
 
 const char *   gnc_table_get_entry (Table *table, VirtualLocation virt_loc);
 
@@ -205,6 +212,8 @@ guint32        gnc_table_get_fg_color (Table *table, VirtualLocation virt_loc);
 
 guint32        gnc_table_get_bg_color (Table *table, VirtualLocation virt_loc,
                                        gboolean *hatching);
+guint32        gnc_table_get_gtkrc_bg_color (Table *table, VirtualLocation virt_loc,
+        gboolean *hatching);
 
 void           gnc_table_get_borders (Table *table, VirtualLocation virt_loc,
                                       PhysicalCellBorders *borders);
@@ -219,11 +228,12 @@ BasicCell *    gnc_table_get_cell (Table *table, VirtualLocation virt_loc);
 
 const char *   gnc_table_get_cell_name (Table *table,
                                         VirtualLocation virt_loc);
-
+const gchar *  gnc_table_get_cell_type_name (Table *table,
+        VirtualLocation virt_loc);
 gboolean       gnc_table_get_cell_location (Table *table,
-                                            const char * cell_name,
-                                            VirtualCellLocation vcell_loc,
-                                            VirtualLocation *virt_loc);
+        const char * cell_name,
+        VirtualCellLocation vcell_loc,
+        VirtualLocation *virt_loc);
 
 void           gnc_table_save_cells (Table *table, gpointer save_data);
 
@@ -244,18 +254,18 @@ void        gnc_table_set_vcell (Table *table, CellBlock *cursor,
 
 /* Set the virtual cell data for a particular location. */
 void        gnc_table_set_virt_cell_data (Table *table,
-                                          VirtualCellLocation vcell_loc,
-                                          gconstpointer vcell_data);
+        VirtualCellLocation vcell_loc,
+        gconstpointer vcell_data);
 
 /* Set the visibility flag for a particular location. */
 void        gnc_table_set_virt_cell_visible (Table *table,
-                                             VirtualCellLocation vcell_loc,
-                                             gboolean visible);
+        VirtualCellLocation vcell_loc,
+        gboolean visible);
 
 /* Set the cellblock handler for a virtual cell. */
 void        gnc_table_set_virt_cell_cursor (Table *table,
-                                            VirtualCellLocation vcell_loc,
-                                            CellBlock *cursor);
+        VirtualCellLocation vcell_loc,
+        CellBlock *cursor);
 
 /* The gnc_table_move_cursor() method will move the cursor (but not
  *   the cursor GUI) to the indicated location. This function is
@@ -275,7 +285,7 @@ void        gnc_table_move_cursor_gui (Table *table, VirtualLocation virt_loc);
  *   cursor and gui to the new position. Returns true if the cell
  *   cursor was repositioned. */
 gboolean    gnc_table_verify_cursor_position (Table *table,
-                                              VirtualLocation virt_loc);
+        VirtualLocation virt_loc);
 
 /* The gnc_table_get_vcell_data() method returns the virtual cell data
  *   associated with a cursor located at the given virtual coords, or
@@ -287,8 +297,8 @@ gpointer    gnc_table_get_vcell_data (Table *table,
  *   be explicitly selected by the user (as opposed to just tabbing
  *   into), are considered valid cells. */
 gboolean    gnc_table_find_close_valid_cell (Table *table,
-                                             VirtualLocation *virt_loc,
-                                             gboolean exact_cell);
+        VirtualLocation *virt_loc,
+        gboolean exact_cell);
 
 /** UI-specific functions *******************************/
 
@@ -299,7 +309,7 @@ void        gnc_table_realize_gui (Table *table);
 
 /* Refresh the current cursor gui */
 void        gnc_table_refresh_current_cursor_gui (Table * table,
-                                                  gboolean do_scroll);
+        gboolean do_scroll);
 
 /* Refresh the whole GUI from the table. */
 void        gnc_table_refresh_gui (Table *table, gboolean do_scroll);
@@ -312,25 +322,38 @@ void        gnc_table_show_range (Table *table,
 /* Refresh the cursor in the given location. If do_scroll is TRUE,
  * scroll the register so the location is in view. */
 void        gnc_table_refresh_cursor_gui (Table * table,
-                                          VirtualCellLocation vcell_loc,
-                                          gboolean do_scroll);
+        VirtualCellLocation vcell_loc,
+        gboolean do_scroll);
 
 /* ==================================================== */
 
 void         gnc_table_wrap_verify_cursor_position (Table *table,
-                                                    VirtualLocation virt_loc);
+        VirtualLocation virt_loc);
 
 gboolean     gnc_table_virtual_loc_valid(Table *table,
-                                         VirtualLocation virt_loc,
-                                         gboolean exact_pointer);
+        VirtualLocation virt_loc,
+        gboolean exact_pointer);
 
 gboolean     gnc_table_move_tab (Table *table,
                                  VirtualLocation *virt_loc,
                                  gboolean move_right);
 
+/** Moves away from virtual location @a virt_loc by @a phys_row_offset physical
+ *  rows. Virtual cells that are not visible are skipped over.
+ *
+ *  @param table The table
+ *
+ *  @param virt_loc The virtual location to start from. If the move succeeds,
+ *  it is updated with the new location.
+ *
+ *  @param phys_row_offset The number of physical rows to move. A positive
+ *  number moves down and a negative number moves up.
+ *
+ *  @return TRUE if the location changed, FALSE otherwise
+ */
 gboolean     gnc_table_move_vertical_position (Table *table,
-                                               VirtualLocation *virt_loc,
-                                               int phys_row_offset);
+        VirtualLocation *virt_loc,
+        int phys_row_offset);
 
 gboolean     gnc_table_enter_update(Table *table,
                                     VirtualLocation virt_loc,
@@ -344,9 +367,9 @@ gboolean     gnc_table_confirm_change(Table *table, VirtualLocation virt_loc);
 
 const char * gnc_table_modify_update(Table *table,
                                      VirtualLocation virt_loc,
-                                     const GdkWChar *change,
+                                     const char *change,
                                      int change_len,
-                                     const GdkWChar *newval,
+                                     const char *newval,
                                      int newval_len,
                                      int *cursor_position,
                                      int *start_selection,
@@ -367,3 +390,6 @@ gboolean     gnc_table_traverse_update(Table *table,
                                        VirtualLocation *dest_loc);
 
 #endif /* TABLE_ALLGUI_H */
+/** @} */
+/** @} */
+/** @} */
